#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <naken_plugin.h>

static struct naken_context_t *context;

/*********************************************************
 *
 * This function is called when Naken Chat starts up and
 * your module in initialized.  The naken_context_t structure
 * holds pointers to the Users structs and all useful
 * functions within Naken Chat.  This structure will give you
 * access to all user info and any text output functions,
 * channel changing functions, and anything else relevant.
 * Return 0 on success or -1 for failure.
 */

int naken_init(struct naken_context_t *naken_context)
{

  // This is needed if you want to call functions in Naken Chat
  context=naken_context;

  return 0;
}

/*********************************************************
 * This function will be called after Naken Chat receives
 * a . command that it doesn't know how to handle.  For
 * example if the user types .$ hello! the $ is an unknown
 * command to Naken Chat so this ID will equal the user who
 * typed .$ hello! and char *text will be "$ hello!".  If
 * this function actually does something with the command it
 * should return a 1.  If it's unused return 0 so Naken Chat
 * knows to do something else with it.
 */

int naken_dot_command(int ID, const char *text)
{

  return 0;
}

/*********************************************************
 * This function is called before sending the text to output
 * part of Naken Chat that displays the text for everyone in
 * the channel.  ID will be the line number of the person who
 * typed it and text will be what they typed.  So for example
 * if [1]mike types "HI EVERYONE!" this function will get
 * ID=1 and char *text="HI EVERYONE!".  This function could
 * turn all the text to lowercase or such.  If this function
 * wants Naken Chat to supress output of this text return -1.
 * This can be used if your plugin has its own command set such
 * as typing ! before the text.  So if you have a plugin that
 * picks up dictionary definitions and you output the result
 * to only the user and you want Naken Chat to then ignore this
 * text since you already processed it, you can return -1 here.
 * Example: char *text="!dictionary book".  You can send output
 * to the user on ID and have Naken Chat forget the user typed
 * this.
 */

int naken_public_text(int ID, char *text)
{
  while(*text!=0)
  {
    *text=tolower(*text);
    text++;
  }

  return 0;
}

/*********************************************************
 * Before Naken Chat outputs text, you can modify it here.
 * Warning here: Don't exceed the length of the output buffer
 * or you can cause a crash or other bad things
 */

void naken_output_filter(char *text)
{


}


