/**
 *  Naken Chat
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPL
 *
 * Copyright 1998-2013 by Michael Kohn
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#ifndef WINDOWS
#include <pthread.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <signal.h>
#include <netdb.h>
#endif
#include <errno.h>
#include <time.h>
#include <string.h>
#ifdef WINDOWS
#include <windows.h>
#include <process.h>
#include <winsock.h>
#endif

#include "buffered_read.h"
#include "chat_thread.h"
#include "general.h"
#include "info.h"
#include "language.h"
#include "message.h"
#include "nakenchat.h"
#include "read_config.h"
#include "version.h"

#ifdef PLUGINS
static int check_dot_plugin(int ID, char *out_string)
{
struct naken_plugins_t *naken_plugin;

  naken_plugin=naken_plugins;
  while(naken_plugin!=0)
  {
    if (naken_plugin->naken_dot_command!=0)
    {
      if (naken_plugin->naken_dot_command(ID,out_string+1)!=0)
      { return 1; }
    }
    naken_plugin=naken_plugin->next_plugin;
  }

  return 0;
}
#endif

void chat_thread(int *params)
{
int t=0,r,trim;
unsigned int tid;
char tempString[1500];
unsigned char ch;
char *bad;
char *badwordptr;
/* char *trimString; */
int ID=0;
int msock=0;
fd_set readset;
struct timeval tv;
char *out_string;
char *s;
time_t gc_time;
int thread_num;
int dirty_buffer;
int gstrlen;
#ifdef PLUGINS
struct naken_plugins_t *naken_plugin;
#endif

  thread_num=params[0];
  gc_time=time(NULL);

  while(1)
  {
    if (time(NULL)-gc_time>GC_TIME)
    {
      for (r=thread_num; r<server_info->max_conn; r=r+MAX_USER_THREADS)
      {
        if (users[r]->inuse>1)
        {
          if (time(NULL)-users[r]->idletime>GC_TIME)
          {
            if (users[r]->socketid!=0)
            {
              socketdie(users[r]->socketid);
              users[r]->socketid=0;
            }

            users[r]->inuse=0;
          }
        }

        if (r<server_info->min_conn) continue;

        if (users[r]->inuse==0 && users[r]->idletime!=-1)
        {
          if (time(NULL)-users[r]->idletime>GC_TIME)
          {
#ifdef DEBUG
if (server_info->debug==1) printf("Deallocating line %d\n",r);
#endif
            free(users[r]->gags);
            free(users[r]);
            users[r]=&nulluser;
          }
        }
      }

      if (server_info->max_idle_time>0)
      {
        for (r=thread_num; r<server_info->max_conn; r=r+MAX_USER_THREADS)
        {
#ifdef REALIDLETIME
          if ((users[r]->inuse!=0 && users[r]->user_level<5) && time(NULL)-users[r]->real_idletime>server_info->max_idle_time)
#else
          if ((users[r]->inuse!=0 && users[r]->user_level<5) && time(NULL)-users[r]->idletime>server_info->max_idle_time)
#endif
          {
            message(r,KICKED_FOR_IDLING,0);
            users[r]->inuse=2;
            kill_user(r);
          }
        }
      }

      gc_time=time(NULL);
    }

/* printf("HERE AGAIN\n"); */

    FD_ZERO(&readset);
/*
    FD_SET(sockfd,&readset);
    msock=sockfd;
*/
    msock=0;
    dirty_buffer=0;

    // FIXME - make this high_conn
    for (r=thread_num; r<server_info->max_conn; r=r+MAX_USER_THREADS)
    {
      if (users[r]->inuse==1 || users[r]->inuse==100)
      {
        FD_SET(users[r]->socketid,&readset);
        if (msock<users[r]->socketid) msock=users[r]->socketid;
        if (users[r]->bptr<users[r]->blen) dirty_buffer=1;
      }
    }

    if (dirty_buffer==0)
    {    
      tv.tv_sec=1;
      tv.tv_usec=0;
    }
      else
    {
      tv.tv_sec=0;
      tv.tv_usec=1;
    }

#ifdef WINDOWS
    if (msock==0)
    {
      _sleep(1000);
      continue;
    }
#endif

    if ((t=select(msock+1,&readset,NULL,NULL,&tv))==-1)
    {
#ifdef WINDOWS
      /* if (WSAGetLastError()!=WSANOTINITIALISED) printf("yes %d\n",errno); */
      if (WSAGetLastError()!=WSAEINTR)
#else
      if (errno!=EINTR)
#endif
      { 
#ifdef DEBUG
        if (server_info->debug==1) printf("not EINTR %d\n",thread_num);
#endif
      }
         else
      { 
#ifdef DEBUG
        if (server_info->debug==1) printf("Problem with select\n"); 
#endif
        continue;
      }
    }

/*
printf("Heartbeat %d  --- %d\n",thread_num,t);
*/

    for (ID=thread_num; ID<server_info->max_conn; ID=ID+MAX_USER_THREADS)
    {
      errno=0;
      if (users[ID]==0) continue;
      if (users[ID]->inuse!=1 && users[ID]->inuse!=100) continue;

/*
printf("Checking line %d     %d %d %d\n",ID,users[ID]->bptr,users[ID]->blen,users[ID]->ptr);
*/
      if (users[ID]->inuse==100 && time(NULL)-users[ID]->logontime>60)
      { kill_user(ID); }

      if (users[ID]->bptr<users[ID]->blen)
      {
        /* printf("dirty\n"); */
      }
        else
      if (FD_ISSET(users[ID]->socketid,&readset)) 
      {  } 
        else 
      continue;

      r=buffered_read(ID);

      if (r<0)
      {
#ifdef DEBUG
        if (server_info->debug==1)
        { printf("Lost Connection (NULL): %d\n",ID); fflush(stdout); }
#endif

        if (users[ID]->inuse==1 || users[ID]->inuse==100) kill_user(ID);
        continue;
      }

      if (r==0) continue;

/*
printf("FILE: %d\n",ID);
fflush(stdout);

printf("READ: %s\n\n",users[ID]->out_string);
fflush(stdout);
*/

      out_string=users[ID]->out_string;
      gstrlen=r;
      while(out_string[gstrlen-1]==' ')
      {
        out_string[gstrlen-1]=0;
        gstrlen--;
      }

      trim=0;

      while(out_string[trim]==' ') trim++;

      if (server_info->censor_level>0 && out_string[trim]!='.' && out_string[trim+1]!='P')
      {
        badwordptr=server_info->bad_word;
        for (r=0; r<server_info->bad_word_count; r++)
        {
          if ((bad=strcasestr(out_string+trim, badwordptr))!=0)
          {
            if (server_info->censor_level==1)
            {
              while (bad[0]!=0 && bad[0]!=' ') { bad[0]='*'; bad++; }
            }
              else
            if (server_info->censor_level==2)
            {
              message(ID,PLEASE_CENSOR,0);
              out_string[trim]=0;
              r=999999;
            }
              else
            if (server_info->censor_level==3)
            {
              users[ID]->warnings++;
              if (users[ID]->warnings<4)
              { message(ID,PLEASE_CENSOR,0); }
                else
              {
                message(ID,WARNED,0);
                users[ID]->inuse=2;
                kill_user(ID);
              }

              out_string[trim]=0;
              r=999999;
            }
              else
            if (server_info->censor_level==4)
            {
              message(ID,PLEASE_CENSOR,0);
              users[ID]->inuse=2;
              kill_user(ID);
              r=999999;
            }

            r=r-1;
            continue;
          }

          badwordptr=badwordptr+strlen(badwordptr)+1;
        }
      }

      if (r==999999) continue;

      if (time(NULL)-users[ID]->idletime<2)
      { if ((out_string[trim]!='\n' && out_string[trim]!='\r') && out_string[trim]!=0) users[ID]->spam++; }
        else
      { users[ID]->spam=0; }

      if (users[ID]->spam==8)
      {
        message(ID,SPAM,0);
        kill_user(ID);
        continue;
      }

      users[ID]->idletime=time(NULL);

#ifdef DEBUG
      if (server_info->debug==1)
      {
        printf("Read in: %d bytes on thread %d.\n",(int)strlen(out_string),thread_num);
        printf("%s typed: %s\n",users[ID]->username,out_string);
        fflush(stdout); 
      }
#endif

      ch=out_string[0];

      if (users[ID]->inuse==100)
      {
        if (ch=='.' || ch=='/')
        {
          if (out_string[1]=='Z')
          {
            users[ID]->flags.userlist^=1;
            continue;
          }
            else
          if (out_string[1]!='n' && out_string[1]!='q')
          {
            message(ID,NAME_YOURSELF,0);
            continue;
          }
        }
      }

      if (ch=='.' || ch=='/')
      {
        ch=out_string[1];

        trim=2;
        while(out_string[trim]==' ' && out_string[trim]!=0) trim++;

        switch(ch)
        {
          case '#':
            sprintf(tempString,">> %d\r\n", MAX_MESSAGE_SIZE-2);
          message(ID,tempString,0);
            continue;

          case 'P':
            if (strcmp(out_string+trim,server_info->sysop_pw)==0)
            {
              /* users[ID]->sysop=users[ID]->sysop^1; */
              if (users[ID]->user_level==10)
              { users[ID]->user_level=0; }
                else
              { users[ID]->user_level=10; }

              if (users[ID]->user_level==0)
              { message(ID,SUPER_OFF,0); }
                else
              { message(ID,SUPER_ON,0); }
            }
              else  
            if (users[ID]->user_level==10)
            {
              r=get_num(out_string+trim,ID,0);

              if (r>=0)
              {
                if (users[r]->user_level==10)
                { users[r]->user_level=0; }
                  else
                { users[r]->user_level=10; }

                if (users[r]->user_level==10)
                { 
                  message(r,GRANT_SUPER,0);
                  sprintf(tempString,NOW_SUPER,r,users[r]->username); 
                }
                  else
                { 
                  message(r,REVOKE_SUPER,0);
                  sprintf(tempString,NOT_SUPER,r,users[r]->username); 
                }
                message(ID,tempString,0);
              }
            }
              else
            { message(ID,UNDERSTAND_MESSAGE,0); }
            continue;

          case 'Z':
            clientshow(ID);
            users[ID]->flags.userlist^=1;
            continue;

          case 'a':
            if (!server_info->flags.channel_disallow || users[ID]->user_level>=8)
            { channelinfo(ID); }
            continue;

          case 'b':
            users[ID]->flags.beeps^=1;

            if (users[ID]->flags.beeps)
            { message(ID,BEEPS_ON,0); }
              else
            { message(ID,BEEPS_OFF,0); }
            continue;

          case 'c':
            if (!server_info->flags.channel_disallow || users[ID]->user_level>=8)
            {
              if ((gstrlen-trim)>0)
              {
                out_string[14+trim]=0;
                r=0;
                while(out_string[r]!='\n' && out_string[r]!='\r' && out_string[r]!=0) r++;
                out_string[r]=0;
                if (r==trim)
                {
                  sprintf(tempString,YOU_ARE_IN,server_info->channels[users[ID]->channel]);
                  message(ID,tempString,0);
                  continue;
                }

                t=0;
                r=channel(ID,out_string+trim);
                if (r==-2)
                { 
                  sprintf(tempString,YOU_ALREADY_IN,out_string+trim);
                  message(ID,tempString,0); 
                }
                  else  
                if (r==-1)
                { message(ID,CHAN_LOCKED,0); }
                  else
                if (r==-3)
                { message(ID,CHAN_FULL,0); }
                  else
                {
                  if (server_info->channel_op[users[ID]->channel]==ID && users[ID]->channel!=r)
                  { server_info->channel_op[users[ID]->channel]=-1; }

#ifdef HIDDEN_CAVES
                  t=conv_num(out_string+trim);
                  if (t>=HIDDEN_CAVES_START && t<=HIDDEN_CAVES_END)
                  { 
                    sprintf(tempString,WENT_TO_CAVE,ID,users[ID]->username);
                    sendchanbut(ID,users[ID]->channel,tempString); 
                   }
                     else
#endif
                  { 
                    sprintf(tempString,WENT_TO_CHAN,ID,users[ID]->username,server_info->channels[r]);
                    sendchanbut(ID,users[ID]->channel,tempString); 
                  }

#ifdef USER_LIST_CHAN
                  update_list(ID,0);
#endif
                  users[ID]->channel=r;
                  //users[ID]->uflags=users[ID]->uflags&65531;
                  users[ID]->flags.squelched=0;
                  sprintf(tempString,HAS_JOINED,ID,users[ID]->username);
                  sendchan(ID,users[ID]->channel,tempString);
#ifdef USER_LIST_CHAN
                  update_list(ID,1);
#endif
                  if (users[ID]->flags.userlist) clientshow(ID);
                }
              }
                else
              {
                sprintf(tempString,YOU_ARE_IN,server_info->channels[users[ID]->channel]);
                message(ID,tempString,0);
              }
            }
              else
            {
              message(ID,UNDERSTAND_MESSAGE,0);
            }
            continue;

          case 'd':
            users[ID]->flags.time_stamps^=1;

            if (users[ID]->flags.time_stamps)
            { message(ID,TIMESTAMP_ON,0); }
              else
            { message(ID,TIMESTAMP_OFF,0); }
            continue;

          case 'e':
            if (out_string[trim]!=0)
            {
              users[ID]->emote_char=out_string[trim];
              sprintf(tempString,NEW_EMOTE_CHAR,users[ID]->emote_char);
              message(ID,tempString,0);
            }
              else
            {
              users[ID]->flags.echos^=1;

              if (users[ID]->flags.echos)
              { message(ID,ECHOS_ON,0); }
                else
              { message(ID,ECHOS_OFF,0); }
            }
            continue;

          case 'f':
            who_short(ID);
            continue;

          case 'g':
            r=get_num(out_string+trim,ID,0);

            if (r>=0)
            {
              if (r==ID)
              { message(ID,CANT_GAG,0); }
                else
              {
                TOGGLE_GAG(ID,r);

                if (IS_GAGGED(ID,r))
                { sprintf(tempString,YOU_GAGGED,r,users[r]->username); }
                  else
                { sprintf(tempString,YOU_UNGAGGED,r,users[r]->username); }
                message(ID,tempString,0);
              }
            }
            continue;

          case 'h':
            if (out_string[2]=='e') help(ID);
              else
            if (out_string[2]=='i')
            {
              users[ID]->flags.hilites^=1;

              if (users[ID]->flags.hilites)
              { message(ID,HILITE_ON,0); }
                else
              { message(ID,HILITE_OFF,0); }
            }
              else
            if (out_string[2]=='u')
            {
              users[ID]->flags.hush_yells^=1;

              if (users[ID]->flags.hush_yells)
              { message(ID,YELLING_OFF,0); }
                else
              { message(ID,YELLING_ON,0); }
            }
              else
            { message(ID,UNDERSTAND_MESSAGE,0); }
            continue;

          case 'i':
            r=get_num(out_string+trim,ID,CAN_BE_NOTHING);
            if (r==-2) continue;
            if (r==-1)
            { r=ID; }

            if ((r<0 || r>=server_info->max_conn) || (users[r]->inuse!=1 
                 || (users[r]->flags.hidden_sysop
                 && users[ID]->user_level<7)))
            { message(ID,NOT_HERE,0); }
              else
            {
              if (server_info->flags.show_ips || users[ID]->user_level>=7)
              { sprintf(tempString,INFO_LOC,r,users[r]->username,users[r]->location); }
                else
              { sprintf(tempString,">> [%d]%s\r\n",r,users[r]->username); }

              strcat(tempString,TIME_ONLINE);
              timediff(time(NULL),users[r]->logontime,out_string);
              strcat(tempString,out_string);
              strcat(tempString,"\r\n");
              strcat(tempString,IDLE_TIME);
              timediff(time(NULL),users[r]->idletime,out_string);
              strcat(tempString,out_string);
              strcat(tempString,"\r\n");
#ifdef REALIDLETIME
              if (users[ID]->user_level>=8)
              {
                strcat(tempString,">> Real Idle Time: ");
                timediff(time(NULL),users[r]->real_idletime,out_string);
                strcat(tempString,out_string);
                strcat(tempString,"\r\n");
              }
#endif
              message(ID,tempString,0);
            }
            continue;

          case 'k':
            if ((server_info->channel_op[users[ID]->channel]==ID && users[ID]->channel!=0) || (users[ID]->user_level==10 && users[ID]->channel!=0))
            {
              r=get_num(out_string+trim,ID,0);

              if (r>=0)
              {
                if (users[r]->channel==users[ID]->channel || users[ID]->user_level==10)
                {
                  t=channel(r,"0");
                  sprintf(tempString,WAS_KICKED,r,users[r]->username);
                  sendchanbut(r,users[ID]->channel,tempString);
                  sprintf(tempString,KICKED_YOU,ID,users[ID]->username);
                  message(r,tempString,0);

                  users[r]->channel=t;
                  //users[r]->uflags=users[r]->uflags&65531;
                  users[r]->flags.squelched=0;
                  sprintf(tempString,HAS_JOINED,r,users[r]->username);
                  sendchan(r,0,tempString);
                }
                  else
                { message(ID,NOT_IN_CHAN,0); }
              }
            }
              else
            { message(ID,CANNOT_KICK,0); }
            continue;

          case 'l':
            if ((server_info->channel_op[users[ID]->channel]!=ID || users[ID]->channel==0) && users[ID]->user_level<10)
            { message(ID,CANNOT_LOCK,0); }
              else
            if (users[ID]->channel==0)
            { message(ID,CANT_LOCK_MAIN,0); }
              else
            {
              server_info->channel_flags[users[ID]->channel]=server_info->channel_flags[users[ID]->channel]^1;
              if ((server_info->channel_flags[users[ID]->channel]&1)==1)
              { sendchan(ID,users[ID]->channel,CHANNEL_LOCKED); }
                else
              { sendchan(ID,users[ID]->channel,CHANNEL_UNLOCKED); }
            }
            continue;

          case 'm':
            users[ID]->flags.system_messages^=1;

            if (users[ID]->flags.system_messages)
            { message(ID,SYSMES_OFF,0); }
              else
            { message(ID,SYSMES_ON,0); }
            continue;

          case 'n':
            if (out_string[trim]!=0)
            {
              if (server_info->flags.no_name_changing && users[ID]->flags.changed_name)
              {
                message(ID,CANNOT_CHANGE_NAME,0);
                continue;
              }

#ifdef PASSWD 
              if (server_info->passwd[0]!=0)
              {
                r=passwd_check(out_string+trim,ID);
                if (r==1)
                {
                  message(ID,NAME_IS_REGISTERED,0);
                  continue;
                }
                  else
                if (r==2 && server_info->flags.force_password)
                {
                  message(ID,MUST_REGISTER,0);
                  continue;
                }
              }
#endif

              if (server_info->flags.kick_dups)
              {
                r=checkname_dup(&out_string[trim],ID);
                if (r!=-1)
                {
                  kill_user(r);
                }
              }

              if (server_info->flags.no_dup_names)
              {
                if (checkname_dup(&out_string[trim],ID)!=-1)
                {
                  message(ID,NAME_IN_USE,0);
                  continue;
                }
              }

              update_list(ID,0);

              safestrncpy(users[ID]->username,out_string+trim,MAX_USER_NAME_LENGTH);
              sprintf(out_string,YOUR_NAME,users[ID]->username);
              message(ID,out_string,0);

              users[ID]->flags.changed_name=1;

              update_list(ID,1);
              if (users[ID]->inuse==100)
              {
                users[ID]->inuse=1;

                if (server_info->flags.show_ips)
                { sprintf(tempString,NAME_LOGIN1,ID,users[ID]->username,users[ID]->location); }
                  else
                { sprintf(tempString,NAME_LOGIN2,ID,users[ID]->username); }

                if (!server_info->flags.channel_disallow)
                { sendall(ID,tempString); }
                  else
                { sendchan(ID,users[ID]->channel,tempString); }

                if (users[ID]->flags.userlist) clientshow(ID);
              }
            }
              else
            { message(ID,ILLEGAL_NAME,0); }
#ifdef WHOFILE
            if (server_info->whofile[0]!=0) write_whofile();
#endif
            continue;

          case 'o':
            r=get_num(out_string+trim,ID,0);

            if (r<0)
            { }
              else
            if ((server_info->channel_op[users[ID]->channel]!=ID || users[ID]->channel==0) && users[ID]->user_level<10)
            { message(ID,NOT_OWNER,0); }
              else
            if (users[ID]->channel==0)
            { message(ID,NO_MAIN_OWNER,0); }
          else
            if (r==ID)
            { message(ID,ALREADY_OWN,0); }
              else
            if (users[r]->channel!=users[ID]->channel)
            { message(ID,NOT_IN_CHAN,0); }
              else
            {
              server_info->channel_op[users[ID]->channel]=r;
              sprintf(tempString,NEW_OWNER,r,users[r]->username);
              sendchan(-1,users[ID]->channel,tempString);
            }
            continue;

          case 'p':
#ifdef REALIDLETIME
            users[ID]->real_idletime=users[ID]->idletime;
#endif
            r=get_num(out_string+trim,ID,CAN_BE_NOTHING);

            if (r==-1) 
            { r=users[ID]->lastp; }
              else
            if (r==-2) continue;
              else
            { trim=trim+strip(out_string+trim); }

            if (r!=-1)
            {
              if ((gstrlen-trim)==0) strcpy(out_string," ");
              if (users[r]->inuse!=1)
              { message(ID,NOT_HERE,0); }
                else
              if (IS_GAGGED(r,ID))
              {  message(ID,GAGGED_YOU,0); }
                else
              if (server_info->flags.channel_disallow && users[ID]->channel!=users[r]->channel && users[ID]->user_level<8)
              { message(ID,NOT_HERE,0); }
                else
              if (out_string[trim]==users[ID]->emote_char)
              {
                trim++;
                while(out_string[trim]==' ' && out_string[trim]!=0)
                { trim++; }
                sprintf(tempString,PRIVATE1,ID,users[ID]->username,out_string+trim);
                sendpriv(r,ID,tempString);
                users[ID]->lastp=r;
              }
                else
              {
                sprintf(tempString,PRIVATE2,ID,users[ID]->username,out_string+trim);
                sendpriv(r,ID,tempString);
                users[ID]->lastp=r;
              }
            }
              else
            { message(ID,UNDERSTAND_MESSAGE,0); }
            continue;

          case 'q':
            message(ID, BYE_BYE,0);
            kill_user(ID);
            continue;

#ifdef PASSWD
          case 'r':
            if (server_info->flags.name_register && users[ID]->user_level>=server_info->register_level)
            {
              r=passwd_reg_name(ID, out_string+trim);
              if (r==0)
              { message(ID,PASSWD_CHANGED,0); }
                else
              { message(ID,PASSWD_NOT_CHANGED,0); }
            }
              else
            { message(ID,PASSWD_NOT_CHANGED,0); }
            continue;
#endif

          case 's':
            if (server_info->channel_op[users[ID]->channel]==ID && users[ID]->channel!=0)
            {
              r=get_num(out_string+trim,ID,0);

              if (r<0)
              { }
                else  
              {
                if (users[r]->channel==users[ID]->channel)
                {
                  if (users[r]->flags.squelched)
                  {   
                    message(r,YOU_UNSQUELCH,0);
                    sprintf(tempString,YOU_UNSQUELCHED,r,users[r]->username);
                    message(ID,tempString,0);
                  }     
                    else
                  {
                    message(r,YOU_SQUELCH,0);
                    sprintf(tempString,YOU_SQUELCHED,r,users[r]->username);
                    message(ID,tempString,0);
                  }

                  users[r]->flags.squelched^=1;
                }     
                  else
                { message(ID,NOT_IN_CHAN,0); }
              }
            }
              else 
            { message(ID,CANT_SQUELCH,0); }
            continue;

          case 't':
            users[ID]->stamptime=time(NULL);
            sprintf(tempString,THE_TIME,ctime((time_t*)&users[ID]->idletime));
            message(ID,tempString,0);
            continue;

          case 'u':
            timediff(time(NULL),server_info->uptime,out_string);
            sprintf(tempString,UPTIME,out_string);
            message(ID,tempString,0);
            continue;

          case 'v':
  /* PLEASE BE KIND AND GIVE CREDIT FOR DEVELOPER'S HARD WORD. */
            sprintf(tempString,">> %s\r\n",VERSION);
            message(ID,tempString,0);
  /* PLEASE DON'T CHANGE THE VERSION LINE.  THANKS. */
            continue;

          case 'w':
            who(ID,out_string+trim);
            continue;

          case 'y':
#ifdef REALIDLETIME
            users[ID]->real_idletime=users[ID]->idletime;
#endif
            if (!server_info->flags.channel_disallow || users[ID]->user_level>=8)
            {
              if (users[ID]->flags.hush_yells)
              { message(ID,CANT_YELL,0); }
                 else
              if (users[ID]->flags.squelched)
              { message(ID,CANT_YELL,0); }
                 else
              {
                if (out_string[trim]==users[ID]->emote_char)
                {
                  trim++;
                  while(out_string[trim]==' ' && out_string[trim]!=0) trim++;
                  sprintf(tempString,"#%d#%s %s\r\n",ID,users[ID]->username,out_string+trim);
                  yell(ID,tempString);
                }
                  else
                {
                  sprintf(tempString,"#%d#%s: %s\r\n",ID,users[ID]->username,out_string+trim);
                  yell(ID,tempString);
                }
              }
            }
            continue;

          default:
            if (users[ID]->user_level!=10)
            {
#ifdef PLUGINS
              if (naken_plugins!=0)
              { if (check_dot_plugin(ID,out_string)!=0) continue; }
#endif
              message(ID,UNDERSTAND_MESSAGE,0);
              continue;
            }
        }

        /* SYSOP STUFF */

        switch(ch)
        {
#ifdef DAMNIT
          case '+':
            for (r=0; r<server_info->max_conn; r++)
            {
              if (users[r]!=0 && users[r]->inuse!=0)
              {
                sprintf(tempString,"%d) %d\r\n",r,users[r]->inuse);
                message(ID,tempString,0);
              }
            }
            continue;
#endif

#ifdef LEET
          case '3':
            server_info->flags.elite_mode^=1;

            if (server_info->flags.elite_mode)
            { message(ID,">> 3l1t3 writing turned on.\r\n",0); }
              else
            { message(ID,">> 3l1t3 writing turned off.\r\n",0); }
            continue;
#endif

          case 'B':
            if (is_number(out_string+trim)==0)
            {
              r=trim;
              while((out_string[r]!=' ' && out_string[r]!='\t') &&
                 ((out_string[r]!='\n' && out_string[r]!='\r'))
                 && out_string[r]!=0) r++;
              out_string[r]=0;

              if (r==trim)
              { message(ID,TO_BAN,0); }
                else
              { 
                if (ban_address(&out_string[trim])==1)
                { message(ID,TOO_MANY_BANS,0); }
                  else
                {
                  sprintf(tempString,TEMP_BANNED,&out_string[trim]);
                  message(ID,tempString,0);
                }
              }
            }
              else
            {
              r=get_num(out_string+trim,ID,NOT_USERID);
              if (r<0) continue;
              if ((s=unban_number(r))==0)
              {
                message(ID,NO_SITE_BANNED,0);
              }
                else
              {
                sprintf(tempString,UN_BANNED,s);
                message(ID,tempString,0);
              }
            }
            continue;

          case 'C':
            r=get_num(out_string+trim,ID,NOT_USERID);

            if (r<0)
            { /* message(ID,UNDERSTAND_MESSAGE,0); */ continue; }
              else
            if (r>4)
            { message(ID,CENSOR_LEVEL,0); }
              else
            {
              server_info->censor_level=r;
              sprintf(tempString,CENSOR_LEVEL_AT,server_info->censor_level);
              message(ID,tempString,0);
            }
            continue;

          case 'D':
            server_info->flags.no_dup_ips^=1;

            if (server_info->flags.no_dup_ips)
            { message(ID,IP_NODUPON,0); }
              else
            { message(ID,IP_NODUPOFF,0); }
            continue;

          case 'F':
            r=get_num(out_string+trim,ID,CAN_BE_NOTHING);

            if (r==-1)
            { show_levels(ID); }
              else
            if (r==-2)
            { continue; }
              else
            {
              trim=trim+strip(out_string+trim);
              t=get_num(out_string+trim,ID,NOT_USERID|CAN_BE_NOTHING);

              if (t==-1)
              {
                sprintf(tempString,USER_LEVEL_IS,r,users[r]->username,users[r]->user_level);
                message(ID,tempString,0);
              }
                else
              {
                /* WTF */
                if (t>10) t=10;
                users[r]->user_level=t;

                sprintf(tempString,USER_LEVEL_CHANGE,r,users[r]->username,t);
                message(ID,tempString,0);
                sprintf(tempString,YOUR_LEVEL_CHANGE,t);
                message(r,tempString,0);
              }
            }
            continue;

          case 'G':
            if (out_string[trim]==0)
            {
              r=-1; tid=-1;
              for (t=0; t<server_info->max_conn; t++)
              {
                if ((users[t]->inuse==1 && users[t]->logontime<tid) &&
                   (users[t]->user_level<10 && users[t]->channel==0))
                {
                  r=t;
                  tid=users[t]->logontime;
                }
              }

              if (r==-1)
              { message(ID,NO_ONE_TO_GRAB,0); }
            }
              else
            { r=get_num(out_string+trim,ID,0); }

            if (r>=0)
            {
              tid=r;
              t=0;

              r=trim;
              /* printf("%s %d '%s'\n",out_string,trim,out_string+trim); */
              while (out_string[r]!=' ' && out_string[r]!=0) r++;
              while (out_string[r]==' ' && out_string[r]!=0) r++;
              /* printf("%s %d '%s'\n",out_string,r,out_string+r); */

              if (out_string[r]!=0)
              { r=channel(tid,out_string+r); }
                else
              { r=channel(tid,server_info->channels[users[ID]->channel]); }

              if (r==-2)
              { message(ID,ALREADY_IN,0); }
                else  
              if (r==-3)
              { message(ID,NO_FREE_CHAN,0); }
                else  
              { 
                /* r=users[ID]->channel; */
                if (server_info->channel_op[users[tid]->channel]==tid && users[tid]->channel!=users[ID]->channel)
                { server_info->channel_op[users[tid]->channel]=-1; }

#ifdef HIDDEN_CAVES
                t=conv_num(out_string+trim);
                if (t>=HIDDEN_CAVES_START && t<=HIDDEN_CAVES_END)
                { 
                  sprintf(tempString,WENT_TO_CAVE,tid,users[tid]->username);
                          sendchanbut(tid,users[tid]->channel,tempString); 
                }
                  else
#endif
                { 
                  sprintf(tempString,WENT_TO_CHAN,tid,users[tid]->username,server_info->channels[r]);
                  sendchanbut(tid,users[tid]->channel,tempString); 
                }

#ifdef USER_LIST_CHAN
                update_list(tid,0);
#endif
                users[tid]->channel=r;
                //users[tid]->uflags=users[tid]->uflags&65531;
                users[tid]->flags.squelched=0;
                sprintf(tempString,HAS_JOINED,tid,users[tid]->username);
                sendchan(tid,users[tid]->channel,tempString);
                /* clientshow(ID); */
#ifdef USER_LIST_CHAN
                update_list(tid,1);
#endif
              }
            }
            continue;

          case 'H':
            server_info->flags.show_ips^=1;

            if (server_info->flags.show_ips)
            { message(ID,IP_ADDRESS_SHOW,0); }
              else
            { message(ID,IP_ADDRESS_HIDE,0); }
            continue;

          case 'I':
            server_info->flags.international^=1;

            if (server_info->flags.international)
            {
              message(ID,INTERNATIONAL_ON,0);
            }
              else
            {
              message(ID,INTERNATIONAL_OFF,0);
            }
            continue;

          case 'K':
            r=get_num(out_string+trim,ID,0);

            if (r>=0)
            { kill_user(r); }
            continue;

          case 'L':
            safestrncpy(users[ID]->location,out_string+trim,32);
            sprintf(tempString,YOUR_LOCATION,users[ID]->location);
            message(ID,tempString,0);
            continue;

          case 'N':
            r=get_num(out_string+trim,ID,0);
            trim=trim+strip(out_string+trim);

            if (r<0) continue;

            if (out_string[trim]!=0)
            {
              update_list(r,0);
              safestrncpy(users[r]->username,out_string+trim,MAX_USER_NAME_LENGTH);
              sprintf(tempString,SYS_CHANGE_NAME,r,users[r]->username);
              message(ID,tempString,0);
              update_list(r,1);
            }
              else
            {
              message(ID,ILLEGAL_NAME,0);
              continue;
            }

#ifdef WHOFILE
            if (server_info->whofile[0]!=0) write_whofile();
#endif
            continue;

#ifdef QQUIT
          case 'Q':
            exit(0);
#endif

          case 'R':
            r=get_num(out_string+trim,ID,CAN_BE_NOTHING|NOT_USERID);

            if (r==-1)
            {
              message(ID,RE_READ,0);
              read_config();
            }
              else
            {
#ifdef PASSWD
              if (r<-1) r=-1;
                else
              if (r>10) r=10;

              server_info->register_level=r;
              sprintf(tempString,USER_REG_LEVEL,server_info->register_level);
              message(ID,tempString,0);
#else
              message(ID,UNDERSTAND_MESSAGE,0);
#endif
            }
            continue;

          case 'S':
            r=get_num(out_string+trim,ID,0);

            if (r<0)
            { continue; }
              else
            if (users[r]->flags.squelched)
            {   
              message(r,YOU_UNSQUELCH,0);
              sprintf(tempString,YOU_UNSQUELCHED,r,users[r]->username);
              message(ID,tempString,0);
            }     
              else
            {
              message(r,YOU_SQUELCH,0);
              sprintf(tempString,YOU_SQUELCHED,r,users[r]->username);
              message(ID,tempString,0);
            }

            users[r]->flags.squelched^=1;
            continue;

          case 'U':
            r=get_num(out_string+trim,ID,CAN_BE_NOTHING);
            trim=trim+strip(out_string+trim);
            sprintf(tempString,"*%s\r\n",out_string+trim);

            if (r==-2) continue;

            if (tempString[3]==0)
            {
              message(ID,UNDERSTAND_MESSAGE,0);
              continue;
            }

            if (r==-1)
            { sendchanbut(ID,users[ID]->channel,tempString); }
              else
            { message(r,tempString,0); }

            message(ID,URL_SENT,0);
            continue;

          case 'V':
            view_bans(ID);
            continue;

          case 'W':
            if (server_info->logfile[0]==0)
            { message(ID,CHAT_LOG_OFF,0); }
              else
            if (server_info->logfp!=NULL)
            {
              message(ID,CHAT_LOG_OFF,0);
              fclose(server_info->logfp);
              server_info->logfp=NULL;
            }
              else
            {
              message(ID,CHAT_LOG_ON,0);
              logfile_on(ID);
            }
            continue;

          case 'X':
            r=get_num(out_string+trim,ID,0);

            if (r>=0)
            {
              ban_address(users[r]->location);
              sprintf(tempString,TEMP_BANNED,users[r]->location);
  /*
              safestrncpy(bans[server_info->bancount++],users[r]->location,29);
              sprintf(tempString,TEMP_BANNED,bans[server_info->bancount-1]);
  */
              message(ID,tempString,0);

              kill_user(r);
            }
            /*  else */
            /* { message(ID,UNDERSTAND_MESSAGE,0); } */
            continue;

          case 'Y':
            if (!users[ID]->flags.hidden_sysop)
            { message(ID,INVIS_SYSOP_ON,0); }
              else
            { message(ID,INVIS_SYSOP_OFF,0); }

            users[ID]->flags.hidden_sysop^=1;
            continue;

          default:
#ifdef PLUGINS
            if (naken_plugins!=0)
            { if (check_dot_plugin(ID,out_string)!=0) continue; }
#endif
            message(ID,UNDERSTAND_MESSAGE,0);
            continue;
        }
      }
        else 
      {
        if (out_string[trim]!=0)
        {
#ifdef PLUGINS
          naken_plugin=naken_plugins;
          while(naken_plugin!=0)
          {
            if (naken_plugin->naken_public_text!=0)
            {
              if (naken_plugin->naken_public_text(ID,out_string+trim)!=0)
              { break; }
            }
            naken_plugin=naken_plugin->next_plugin;
          }
          if (naken_plugin!=0) continue;
#endif

          if (!users[ID]->flags.squelched)
          {
            if (out_string[trim]==users[ID]->emote_char)
            {  
              trim++;
              while(out_string[trim]==' ' && out_string[trim]!=0) trim++;

#ifdef ENGLISH
              if (out_string[trim]=='\'' && out_string[trim+1]=='s')
              {
                sprintf(tempString,"(%d)%s%s\r\n",ID,users[ID]->username,out_string+trim);
              }
                else
#endif
              {
                sprintf(tempString,"(%d)%s %s\r\n",ID,users[ID]->username,out_string+trim);
              }
            }
              else
            {
              sprintf(tempString,"[%d]%s: %s\r\n",ID,users[ID]->username,out_string+trim);
            }

#ifdef REALIDLETIME
            users[ID]->real_idletime=users[ID]->idletime;
#endif

            sendchan(ID,users[ID]->channel,tempString);
          }
            else
          { message(ID,SQUELCHED,0); }
        }
      }
    }
  }
}

