/**
 *  Naken Chat
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPL
 *
 * Copyright 1998-2013 by Michael Kohn
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#ifndef WINDOWS
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <signal.h>
#include <netdb.h>
#endif
#include <errno.h>
#include <time.h>
#include <ctype.h>
#ifdef WINDOWS
#include <windows.h>
#endif

#include "general.h"
#include "language.h"
#include "message.h"
#include "nakenchat.h"

void safestrncpy(char *dest, char *source, int n)
{
int r;

  for (r=0; r<n; r++)
  { 
    dest[r]=source[r]; 
    if (source[r]==0) break;
  }  

  dest[r]=0;
}

int socketdie(int socketid)
{
int i;

  i=shutdown(socketid,SHUT_RDWR);
#ifdef WINDOWS
  closesocket(socketid);
#else
  close(socketid);
#endif

  return i;
}

int logfile_on(int ID)
{
  server_info->logfp=fopen(server_info->logfile,"a");
  if (server_info->logfp==NULL)
  {
    if (ID!=-1)
    { message(ID,CHAT_LOG_CANT_OPEN,0); }
      else
    { printf("Cannot open chat log for writing!\n"); }
  }

  return 0;
}

#ifdef LEET

/* 13374 patch donated by Oliver Hillmann (tequila :) */

void leet(unsigned char *buf)
{
char lame_dict[] = "elaszotg";
char leet_dict[] = "314z2076";
int r, i, j, l;

  l=strlen(lame_dict);
  r=strlen(buf);

  for (i = 0; i < r; i++) 
  {

    buf[i] = (char)tolower((int)buf[i]);

    for (j = 0; j < l; j++)
    {
      if (buf[i] == lame_dict[j])
      {
        buf[i] = leet_dict[j];
        break;
      }
    }
  }    
}

#endif

#ifdef NOCASESTR
/* OMG */
char *strcasestr(const char *hejstack, const char *needle)
{
int hejlen,neelen;
int t,r;

  hejlen=strlen(hejstack);
  neelen=strlen(needle);
  r=hejlen-neelen;

  for(t=0; t<=r; t++)
  {
    if (strncasecmp(hejstack+t,needle,neelen)==0)
    { return hejstack+t; }
  }

  return 0;
}
#endif

void destroy()
{

  if (server_info->logfp!=NULL) fclose(server_info->logfp);

#ifdef DEBUG
  if (server_info->debug==1)
  { printf("Chat server terminated.\n"); }
#endif

#ifndef WINDOWS
  kill(0,SIGKILL);
#endif
  exit(0);
}

#ifndef WINDOWS
void broken_pipe(int signal_num)
{
#ifdef DEBUG
  if (server_info->debug==1) printf("Signal thrown %d\n",signal_num);
#endif

  setsignals();
}

void setsignals()
{
  signal(SIGPIPE,broken_pipe);
  signal(SIGURG,broken_pipe);
  signal(SIGIO,broken_pipe);
  signal(SIGHUP,broken_pipe);

  signal(SIGINT,destroy);
  signal(SIGTERM,destroy);
}
#endif

static char *collapse(char *d, int c, char *old)
{
char *s;

  s=d;
  while(s[0]!=0) s++;
  s++;
  while(1)
  {
    if (c<=0) break;
    d[0]=s[0];
    if (d[0]==0) c--;
    d++; s++;
  }
  d++;
  strcpy(d,old);
  return d;
}

char *unban_number(int i)
{
char *bans;
char temp[256];
int r;

  bans=server_info->bans;
  for (r=0; r<server_info->ban_count; r++)
  {
    if (r==i)
    {
      server_info->ban_count--;
      strcpy(temp,bans);
      return collapse(bans,server_info->ban_count-i,temp);
    }

    bans=bans+strlen(bans)+1;
  }

  return NULL;
}

#if 0
int list_remove_by_index(char *list, int len, int index)
{

  return -1;
}

int list_append(char *list, int len, char *token)
{

  return -1;
}

int list_find(char *list, char *token)
{
int ptr;
int index=0;

  while(*list!=0)
  {
    ptr=0;
    while(1)
    {
      if (token[ptr]==0)
      {
        if (*list==0) return index;
        while(*list!=0) list++;
        break;
      }

      list++;
      ptr++;
    }

    index++;
  }

  return -1;
}
#endif

int ban_address(char *addr)
{
char *banptr;
int r;

  banptr=server_info->bans;
  for (r=0; r<server_info->ban_count; r++)
  {
    while(banptr[0]!=0) banptr++;
    banptr++;
  }

  if (banptr-server_info->bans+strlen(addr)+1>BANS_BUFFER_SIZE)
  { return 1; }
  strcpy(banptr,addr);
  server_info->ban_count++;

  return 0;
}

int add_to_list(char *list, char *s, int list_len)
{
int len;
int ptr;

  len=strlen(s);
  ptr=0;

  while(ptr<list_len)
  {
    if (list[ptr]==0) break;
    ptr=ptr+list[ptr]+1;
  }

  if (ptr+len+2>list_len) return -1;

  list[ptr]=len;
  memcpy(list+ptr+1,s,len);
  list[ptr+len+1]=0;

  return 0;
}

int search_list(char *list, char *s)
{
int len;
int ptr;

  len=strlen(s);
  ptr=0;

  while(1)
  {
    if (list[ptr]==0) break;
    if (list[ptr]==len)
    {
      if (memcmp(list+ptr+1,s,len)==0) return 1;
    }
    ptr=ptr+list[ptr]+1;
  }

  return 0;
}

int get_num(char *s, int ID, int flags)
{
int r,n=0;

  r=0;
  while (s[r]!=0 && (s[r]>='0' && s[r]<='9'))
  {
    if (r<8) n=n*10+(s[r]-'0');
    r++;
  }

  if (r==0) 
  {
    if ((flags&CAN_BE_NOTHING)==0)
    { message(ID,UNDERSTAND_MESSAGE,0); }
    return -1;
  }

  if ((flags&NOT_USERID)==0)
  {
    if (n>=server_info->max_conn || users[n]==0 || users[n]->inuse!=1)
    {
      if (ID>=0) message(ID,NOT_HERE,0);
      return -2;
    }
  }

  return n;
}

int strip(char *s)
{
int r=0;

   while(s[r]>='0' && s[r]<='9') r++;
   while(s[r]==' ') r++;

   return r;
}

void timediff(time_t time1, time_t time2, char *thisString)
{
int temp;
char tempString[70];

  thisString[0]=0;
  if (time1<time2) { temp=time1; time1=time2; time2=temp; }
  temp=time1-time2;
  if (temp!=0) 
  { sprintf(tempString,"%dsec",temp%60); temp=temp/60; strcpy(thisString,tempString); }
  if (temp!=0)
  { sprintf(tempString,"%dmin %s",temp%60,thisString); temp=temp/60; strcpy(thisString,tempString); }
  if (temp!=0)
  { sprintf(tempString,"%dhours %s",temp%24,thisString); temp=temp/24; strcpy(thisString,tempString); }
  if (temp!=0)
  { sprintf(tempString,"%ddays %s",temp%365,thisString); temp=temp/365; strcpy(thisString,tempString); }
  if (temp!=0)
  { sprintf(tempString,"%dyears %s",temp,thisString); strcpy(thisString,tempString); }
}

void shorttimediff(time_t time1, time_t time2, char *thisString)
{
int temp;
char unit='s';
int len=0;

  thisString[0]=0;

  if (time1<time2)
  { temp=time2-time1; }
    else
  { temp=time1-time2; }

  // len=temp%60;
  len=temp;
  temp=temp/60;
  if (temp!=0)
  {
    unit='m';
    // len=temp%60;
    len=temp;
    temp=temp/60;
    if (temp!=0)
    {
      unit='h';
      // len=temp%24;
      len=temp;
      temp=temp/24;
      if (temp!=0)
      {
        unit='d';
        // len=temp%365;
        len=temp;
        temp=temp/365;
        if (temp!=0)
        {
          unit='y';
          len=temp;
        }
      }
    }
  }

  if (len==0 && unit=='s') return;
  sprintf(thisString,"%d%c",len,unit);
}

int channel(int ID, char *orig_name)
{
int r,new_channel;
char channel_name[CHANNEL_NAME_LEN];

  safestrncpy(channel_name,orig_name,CHANNEL_NAME_LEN-1);

  if (strcmp(channel_name,"0")==0) 
  { strcpy(channel_name,MAINCHANNEL); }

  /* Check if user is already in this channel */
  if (strcasecmp(channel_name,server_info->channels[users[ID]->channel])==0) return -2;

  /* Check if the channel is locked */
  new_channel=-1;

  /* Check if channel exists already */

  for (r=0; r<MAXCHAN; r++)
  { 
    if (server_info->channels[r][0]==0) new_channel=-2;
    if (strcasecmp(channel_name,server_info->channels[r])==0)
    {
      /* If channel exists and is locked, don't let user in */
      if ((server_info->channel_flags[r]&1)==1 && users[ID]->user_level<10) return -1;
      new_channel=r;
      break;
    }
  }
   
  if (users[ID]->channel!=0 && new_channel!=-1)
  {
    for (r=0; r<server_info->max_conn; r++)
    {
      if (users[r]->inuse==1)
      {
        if (users[r]->channel==users[ID]->channel && r!=ID) r=server_info->max_conn;
      }
    }

    /* if (r<MAXCHAN+2) */
    if (r==server_info->max_conn)
    {
      server_info->channels[users[ID]->channel][0]=0;
      server_info->channel_op[users[ID]->channel]=-1;
      server_info->channel_flags[users[ID]->channel]=0;
    }
  }

  if (new_channel>=0) return new_channel;
    else
  if (new_channel==-1) return -3;

  for (r=1; r<MAXCHAN; r++)
  {
    if (server_info->channels[r][0]==0)
    {
      strcpy(server_info->channels[r],channel_name);
      if (!server_info->flags.channel_disallow && search_list(server_info->no_owner,channel_name)==0)
      { server_info->channel_op[r]=ID; }
        else
      { server_info->channel_op[r]=-1; }
      return r;
    }
  }

  return users[ID]->channel;
}

int conv_num(char *num)
{
int r,n;
unsigned char *s=(unsigned char *)num;

  n=0; r=0;

  while(r<10)
  {
    if (s[r]<'0' || s[r]>'9') break;
    n=(n*10)+(s[r]-'0');
    r++;
  }
  if (r==0) return -1;

  return n;
}

int is_number(char *s)
{
char *start=s;

  while(*s!=' ' && *s!='\t' && *s!=0)
  {
    if (*s<'0' || *s>'9') return 0;
    s++;
  }

  return s==start?0:1;
}

/* ACHTUNG! Danka schon Oliver Hillmann fur zis funktion! :)
 * unsigned int sql_esc(const char *s, char *r)
 * input: string (const char *) to a string which should be escaped
 * output: string (char *r) with the resulting escaped string)
 * contraints: r must provide at least 2*strlen(s) allocated memory
 * return value:
 * 0: escaping hasnt taken place
 * 1: escaping has happened
 */
unsigned int sql_esc(const char *s, char *r)
{
        unsigned int i = 0;
        unsigned int j = 0;
        unsigned int escaped = 0;
        while (s[i] != '\0')
        {
                switch (s[i])
                {
                case '\\':
                case '\'':
                        r[j++] = '\\';
                        escaped = 1;
                        break;
                }
                r[j++] = s[i++];
        }
        r[j] = '\0';
        return escaped;
}


