/**
 *  Naken Chat
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPL
 *
 * Copyright 1998-2013 by Michael Kohn
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#ifndef WINDOWS
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <signal.h>
#include <netdb.h>
#endif
#include <errno.h>
#include <time.h>
#include <ctype.h>
#ifdef WINDOWS
#include <windows.h>
#endif

#include "general.h"
#include "language.h"
#include "message.h"
#include "nakenchat.h"

char help_message[sizeof(HELP)+sizeof(CHAN_HELP)+2];

void channelinfo(int ID)
{
char myString[WHO_BUFFER_SIZE];
char tempString[128];
char cops[128];
int r,buffer_ptr;
#ifdef HIDDEN_CAVES
int t;
#endif
 
  buffer_ptr=0;
  strcpy(myString,CHANNEL_HEADER1);
  buffer_ptr=sizeof(CHANNEL_HEADER1)-1;
  strcpy(&myString[buffer_ptr],"------------------------------------------\r\n");
  buffer_ptr+=sizeof("------------------------------------------\r\n")-1;

  strcpy(&myString[buffer_ptr],CHANNEL_HEADER2);
  buffer_ptr+=sizeof(CHANNEL_HEADER2)-1;

  for(r=1; r<MAXCHAN; r++)
  { 
    if (server_info->channels[r][0]!=0)
    {
#ifdef HIDDEN_CAVES
      t=conv_num(server_info->channels[r]);
      if (t<HIDDEN_CAVES_START || t>HIDDEN_CAVES_END)
#endif
      {
        if (users[ID]->user_level<10 && server_info->channel_op[r]>=0) 
        {
          if (users[server_info->channel_op[r]]->user_level==10 && users[server_info->channel_op[r]]->flags.hidden_sysop)
          { continue; }
        }

        if (server_info->channel_op[r]>=0)
        { sprintf(cops,"[%d]%s",server_info->channel_op[r],users[server_info->channel_op[r]]->username); }
          else
        { strcpy(cops,NOONE); }

        if ((server_info->channel_flags[r]&1)==1)
        { sprintf(tempString,"%-15s %-20s "YES"\r\n",server_info->channels[r],cops); }
          else
        { sprintf(tempString,"%-15s %-20s "NO"\r\n",server_info->channels[r],cops); }

	strcpy(&myString[buffer_ptr],tempString);
        buffer_ptr=buffer_ptr+strlen(tempString);

        if (buffer_ptr>WHO_BUFFER_SIZE-150)
        {
          message(ID,myString,0);
          buffer_ptr=0;
          myString[0]=0;
        }
      }
    }
  }
  strcpy(&myString[buffer_ptr],"\r\n");
  message(ID,myString,0);
}

void view_bans(int ID)
{
char myString[BANS_BUFFER_SIZE+1024];
char tempString[256];
char *banptr;
int r,c;

  strcpy(myString,CURRENTBAN);
  banptr=server_info->bans;
  c=0;

  for (r=0; r<server_info->ban_count; r++)
  {
    sprintf(tempString,">> %d: %s\r\n",r,banptr);
    strcat(myString,tempString);
    banptr=banptr+(strlen(banptr))+1;
    c=c+strlen(tempString);
    if (c>BANS_BUFFER_SIZE)
    {
      message(ID,myString,0);
      tempString[0]=0;
      c=0;
    }
  }

  sprintf(tempString,BANCOUNT,server_info->ban_count);
  strcat(myString,tempString);
  message(ID,myString,0);
}

void clientshow(int ID)
{
char myString[server_info->max_conn*(MAX_USER_NAME_LENGTH+10)+4];
char temp[80];
int r;

  strcpy(myString,"@\r\n");

  for (r=0; r<server_info->max_conn; r++)
  {
    if (users[ID]->user_level<10 && (users[r]->user_level==10 && users[r]->flags.hidden_sysop))
    { continue; }

#ifdef USER_LIST_CHAN
    if (users[r]->inuse==1 && users[r]->channel==users[ID]->channel)
#else
    if (users[r]->inuse==1)
#endif
    {
      sprintf(temp,"+[%d]%s\r\n",r,users[r]->username);
      strcat(myString,temp);
    }
  }
  message(ID,myString,0);
}

void show_levels(int ID)
{
char myString[WHO_BUFFER_SIZE];
char tempString[128];
int r,buffer_ptr,k;

  buffer_ptr=0;

  strcpy(myString,"----------------------------------\r\n");
  k=strlen(myString);

  for (r=0; r<server_info->max_conn; r++)
  {
    if (users[r]->inuse==1)
    {
      sprintf(tempString,"[%d]%-20s  %d\r\n",r,users[r]->username,users[r]->user_level);
      strcat(myString,tempString);
      buffer_ptr=buffer_ptr+k;
    }

    if (buffer_ptr>WHO_BUFFER_SIZE-150)
    {
      message(ID,myString,0); 
      buffer_ptr=0;
      myString[0]=0;
    }
  }
  strcat(myString,"----------------------------------\r\n");
  message(ID,myString,0);
}

void who(int ID, char *grep)
{
int c,r,num=-2,trim=0,buffer_ptr;
char myString[WHO_BUFFER_SIZE];
char tempString[1024];
char flags[6];
char channelString[CHANNEL_NAME_LEN];
char location[21];
char idleString[40];
char tempf[60];
time_t curr_time;
#ifdef HIDDEN_CAVES
int n;
#endif

  sprintf(tempf,"%%-%ds %%-14s %%s ",MAX_USER_NAME_LENGTH+9);
  sprintf(myString,tempf,WHO_NAME,WHO_CHANNEL,WHO_IDLE);

  if (server_info->flags.show_ips || users[ID]->user_level==10)
  { strcat(myString,WHO_LOCATION); }
  strcat(myString,"\r\n");

  sprintf(tempf,"[%%d]%%-%ds %%-5s %%-14s %%4s %%s\r\n",MAX_USER_NAME_LENGTH);
  strcpy(location,"");
  curr_time=time(NULL);
  while (grep[trim]==' ') trim++;
  if (is_number(&grep[trim])==1) 
  { num=get_num(&grep[trim],ID,NOT_USERID|CAN_BE_NOTHING); grep[trim]=0; }
    else
  {
    r=trim;
    while((grep[r]!='\n' && grep[r]!='\r') &&
           ((grep[r]!=' ' && grep[r]!='\t') && grep[r]!=0)) r++;
    grep[r]=0;
  }

  c=0;
  buffer_ptr=strlen(myString);

  for(r=0; r<server_info->max_conn; r++)
  {
    if (users[r]->inuse!=1) continue;

    if (users[ID]->user_level<10 && (users[r]->user_level==10 && 
        users[r]->flags.hidden_sysop))
    { continue; }

    if (server_info->flags.channel_disallow &&
        (users[ID]->channel!=users[r]->channel &&
         users[ID]->user_level<8)) continue;

    if (num!=-2 && num!=r) continue;
    strcpy(flags,"-----");

    if (IS_GAGGED(r,ID)) flags[0]='G';
      else
    if (users[r]->flags.hush_yells) flags[0]='H';

    if (IS_GAGGED(ID,r)) flags[1]='g';
      else
    if (users[r]->flags.squelched) flags[4]='S';
      else
    if (users[r]->flags.beeps) flags[1]='B';

    if ((server_info->channel_op[users[r]->channel]==r) && users[r]->channel!=0) flags[2]='O';
    if ((server_info->channel_flags[users[r]->channel]&1)==1) flags[3]='L';
    if (users[r]->flags.ssl_connected) flags[4]='E';

#ifdef HIDDEN_CAVES
    n=conv_num(server_info->channels[users[r]->channel]);
    if (n>=HIDDEN_CAVES_START && n<=HIDDEN_CAVES_END)
    { strcpy(channelString,"Hidden Cave"); }
      else
#endif
    { strcpy(channelString,server_info->channels[users[r]->channel]); }

    shorttimediff(curr_time,users[r]->idletime,idleString);

    if (server_info->flags.show_ips || users[ID]->user_level==10)
    { safestrncpy(location,users[r]->location,20); }

    sprintf(tempString,tempf,r,users[r]->username,flags,channelString,
	idleString,location);

    if (grep[trim]==0)
    {
      strcpy(&myString[buffer_ptr],tempString);
      buffer_ptr=buffer_ptr+strlen(tempString);
      c++;
    }
      else
    {
      if (strstr(tempString,&grep[trim])!=NULL)
      {
        strcpy(&myString[buffer_ptr],tempString);
        buffer_ptr=buffer_ptr+strlen(tempString);
        c++;
      }
    }

    if (buffer_ptr>WHO_BUFFER_SIZE-150)
    {
      message(ID,myString,0); 
      buffer_ptr=0;
      myString[0]=0;
    }
  }

  strcpy(&myString[buffer_ptr],"--------------------------------------------------------");
  buffer_ptr=buffer_ptr+sizeof("--------------------------------------------------------")-1;
  sprintf(tempString,WHO_TOTAL,c);
  strcpy(&myString[buffer_ptr],tempString);

  message(ID,myString,0); 
}

#ifdef WHOFILE
int write_whofile()
{
FILE *out;
char idleString[40];
char location[21];
char channelString[CHANNEL_NAME_LEN];
time_t curr_time;
int r;

#ifndef WINDOWS
  pthread_mutex_lock(&server_info->pthread_mutex);
#else
  EnterCriticalSection(&server_info->win_mutex);
/*
  HANDLE mutex_handle;
  struct SECURITY_ATTRIBUTES mutex_attr;
  mutex_attr.nLength=izeof(SECURITY_ATTRIBUTES);
  mutex_attr.lpSecurityDescriptor=null;
  mutex_attr.bInheritHandle=false;

  mutex_handle=CreateMutex(&mutex_attr, true, null); 
*/
#endif

  out=fopen(server_info->whofile,"w");
  if (out==NULL)
  {
    server_info->whofile[0]=0;
    return -1;
  }

  curr_time=time(NULL);

  for(r=0; r<server_info->max_conn; r++)
  {
    if (users[r]->inuse==1)
    {
      shorttimediff(curr_time,users[r]->idletime,idleString);

      safestrncpy(location,users[r]->location,20);

#ifdef HIDDEN_CAVES
      n=conv_num(server_info->channels[users[r]->channel]);
      if (n>=HIDDEN_CAVES_START && n<=HIDDEN_CAVES_END)
      { strcpy(channelString,"Hidden Cave"); }
	else
#endif
      { strcpy(channelString,server_info->channels[users[r]->channel]); }

      fprintf(out,"[%d]%s\t%s\t%s\t%s\n",r,users[r]->username,channelString,
	idleString,location);

    }
  }

  fclose(out);

#ifndef WINDOWS
  pthread_mutex_unlock(&server_info->pthread_mutex);
#else
  LeaveCriticalSection(&server_info->win_mutex);
  /* CloseHandle(mutex_handle); */
#endif

  return 0;
}
#endif

void who_short(int ID)
{
char myString[WHO_BUFFER_SIZE];
char tempString[256];
int c,r,buffer_ptr;

  buffer_ptr=0;

  strcpy(myString,"--------------------------------------------------------\r\n");
  buffer_ptr=sizeof("--------------------------------------------------------\r\n")-1;

  c=0;
  for (r=0; r<server_info->max_conn; r++)
  {
    if (users[r]->inuse==1)
    {
      if (users[ID]->user_level<10 && (users[r]->user_level==10 && users[r]->flags.hidden_sysop))
      { continue; }

    if (server_info->flags.channel_disallow &&
        (users[ID]->channel!=users[r]->channel &&
         users[ID]->user_level<8)) continue;

      c++;
      sprintf(tempString,"[%d]%-18s",r,users[r]->username);
      strcpy(&myString[buffer_ptr],tempString);
      buffer_ptr=buffer_ptr+strlen(tempString);
      if (c%3==0)
      { strcat(&myString[buffer_ptr],"\r\n"); buffer_ptr=buffer_ptr+2; }
    }

    if (buffer_ptr>WHO_BUFFER_SIZE-150)
    {
      message(ID,myString,0);
      buffer_ptr=0;
      myString[0]=0;
    }
  }
  if (c%3!=0)
  { strcpy(&myString[buffer_ptr],"\r\n"); buffer_ptr=buffer_ptr+2; }

  strcpy(&myString[buffer_ptr],"--------------------------------------------------------\r\n");
  message(ID,myString,0);
}

void help(int ID)
{
  message(ID,help_message,0);
}


