/**
 *  Naken Chat
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPL
 *
 * Copyright 1998-2013 by Michael Kohn
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#ifndef WINDOWS
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <signal.h>
#include <netdb.h>
#endif
#include <errno.h>
#include <time.h>
#include <ctype.h>
#ifdef WINDOWS
#include <windows.h>
#endif

#include "language.h"
#include "nakenchat.h"

static inline int send_message(int ID, char *text, int len)
{
#ifdef ENABLE_SSL
  if (users[ID]->flags.ssl_connected==1)
  {
    return SSL_write(users[ID]->ssl, text, len);
  }
    else
  {
    return send(users[ID]->socketid, text, len, 0);
  }
#else
  return send(users[ID]->socketid, text, len, 0);
#endif
}

void message(int ID, char *text, int hb_flag) /* hb_flag = highlight/beep */
{
char new_text[MAX_MESSAGE_SIZE+64];
// char temp_text[MAX_MESSAGE_SIZE+64];
char temp_text[64];
char *out_text;
time_t ti;
#ifdef PLUGIN
struct naken_plugins_t *naken_plugin=naken_plugins;
#endif

  if (users[ID]->inuse!=1 && users[ID]->inuse!=100) return;

#ifdef PLUGIN
  while(naken_plugin!=0) 
  {
    if (naken_plugin->output_filter!=0)
    {
      naken_plugin->naken_output_filter(text);
    }
    naken_plugin=naken_plugin->next_plugin;
  }
#endif

  if (hb_flag==1)
  {
    if (users[ID]->flags.beeps)
    {
      out_text=new_text;
      if (users[ID]->flags.hilites)
      { sprintf(out_text,"%c%c[1m%s%c[0m",7,27,text,27); }
        else
      { sprintf(out_text,"%c%s",7,text); }
    }
      else
    {
      // strcpy(new_text,text);
      if (users[ID]->flags.hilites)
      {
        out_text=new_text;
        sprintf(out_text,"%c[1m%s%c[0m",27,text,27);
      }
        else
      { out_text=text; }
    }

    if (users[ID]->flags.time_stamps && time(NULL)-users[ID]->stamptime>600)
    { 
      ti=time(NULL);
      sprintf(temp_text,">> %s\r\n",ctime((time_t*)&ti));
      strcat(out_text,temp_text);
      users[ID]->stamptime=time(NULL);
    }

#ifdef LEET
    if (SERVER_ISSET(SERVER_ELITE_MODE))
    { leet(out_text); }
#endif

    if (send_message(ID, out_text, strlen(out_text))<=0)
    {
      users[ID]->inuse=2;
      kill_user(ID);
    }
  }
    else
  { 
    if (users[ID]->flags.time_stamps && time(NULL)-users[ID]->stamptime>600)
    { 
      ti=time(NULL);
      sprintf(new_text,"%s>> %s\r",text,ctime((time_t*)&ti));

#ifdef LEET
      if (SERVER_ISSET(SERVER_ELITE_MODE))
      { leet(new_text); }
#endif

      users[ID]->stamptime=time(NULL);
      if (send_message(ID, new_text, strlen(new_text))<=0)
      {
        users[ID]->inuse=2;
        kill_user(ID);
      }
    }
      else
    {
#ifdef LEET
      if (SERVER_ISSET(SERVER_ELITE_MODE))
      { leet(text); }
#endif

      if (send_message(ID, text, strlen(text))<=0)
      {
        users[ID]->inuse=2;
        kill_user(ID);
      }
    }
  }
}

void sendpriv(int ID_TO, int ID_FROM, char *text)
{
char tempString[MAX_MESSAGE_SIZE+128];

#ifdef DEBUG
  if (server_info->debug==1) printf("TO: %d FROM: %d  MESSAGE: %s\n",ID_TO,ID_FROM,text);
#endif
  if (server_info->logfp!=NULL) 
  {
#ifdef LOG_TIMESTAMPS
    time_t t=time(NULL);
    char stamp[32];
    ctime_r(&t,stamp);
    stamp[strlen(stamp)-1]=0;
    fprintf(server_info->logfp,"%s> TO: %d %s FROM: %d %s  MESSAGE: %s",
            stamp,
            ID_TO,users[ID_TO]->username,
            ID_FROM,users[ID_FROM]->username,text);
#else
    fprintf(server_info->logfp,"TO: %d %s FROM: %d %s  MESSAGE: %s",
            ID_TO,users[ID_TO]->username,
            ID_FROM,users[ID_FROM]->username,text);
#endif
    fflush(server_info->logfp);
  }

  if (ID_TO<server_info->max_conn)
  {
    if (users[ID_TO]->inuse==1)
    {
      message(ID_TO,text,1);

      if (!users[ID_FROM]->flags.echos)
      {
        sprintf(tempString,PRIVATE,ID_TO,users[ID_TO]->username);
      }
	else
      {
        sprintf(tempString,PRIVATE_ECHO,ID_TO,users[ID_TO]->username,text);
      }

      message(ID_FROM,tempString,0);
    }
      else
    { message(ID_FROM,NOT_HERE,0); }
  }
    else
  { message(ID_FROM,NOT_HERE,0); }
}

void sendchan(int ID, int channel, char *text)
{
int r;

  if (server_info->logfp!=NULL) 
  { 
#ifdef LOG_TIMESTAMPS
    time_t t=time(NULL);
    char stamp[32];
    ctime_r(&t,stamp);
    stamp[strlen(stamp)-1]=0;
    fprintf(server_info->logfp,"%s> [%s] %s",stamp,server_info->channels[channel],text); 
#else
    fprintf(server_info->logfp,"[%s] %s",server_info->channels[channel],text); 
#endif
    fflush(server_info->logfp);
  }

  for (r=0; r<server_info->max_conn; r++)
  {
    if (users[r]->inuse==1 && users[r]->channel==channel) 
    {
      if (ID==-1) message(r,text,0);
	else
      // if ((users[r]->gags[ID/8]&(1<<(ID%8)))==0) 
      if (NOT_GAGGED(r,ID)) 
      { 
        if (users[r]->user_level<10 &&
             (users[ID]->user_level==10 &&
             (users[ID]->flags.hidden_sysop)))
        { continue; }

        message(r,text,0);
      }
    }
  }
}

void yell(int ID, char *text)
{
int r;

  if (server_info->logfp!=NULL) 
  { 
#ifdef LOG_TIMESTAMPS
    time_t t=time(NULL);
    char stamp[32];
    ctime_r(&t,stamp);
    stamp[strlen(stamp)-1]=0;
    fprintf(server_info->logfp,"%s> %s",stamp,text); 
#else
    fprintf(server_info->logfp,"%s",text); 
#endif
    fflush(server_info->logfp);
  }

  for(r=0; r<server_info->max_conn; r++)
  {
    if(users[r]->inuse==1)
    {
      // if ((users[r]->gags[ID/8]&(1<<(ID%8)))==0)
      if (NOT_GAGGED(r,ID))
      {
	if (!users[r]->flags.hush_yells) message(r,text,0);
      }
    }
  }
}

void sendchanbut(int ID, int channel, char *text)
{
int r;

  if (server_info->logfp!=NULL) 
  { 
#ifdef LOG_TIMESTAMPS
    time_t t=time(NULL);
    char stamp[32];
    ctime_r(&t,stamp);
    stamp[strlen(stamp)-1]=0;
    fprintf(server_info->logfp,"%s> [%s] %s",stamp,server_info->channels[channel],text); 
#else
    fprintf(server_info->logfp,"[%s] %s",server_info->channels[channel],text); 
#endif
    fflush(server_info->logfp);
  }

  for (r=0; r<server_info->max_conn; r++)
  {
    if ((users[r]->inuse==1 && users[r]->channel==channel) && 
        (r!=ID && (!users[r]->flags.system_messages)))
    { 
      if (users[r]->user_level<10 && (users[ID]->user_level==10 && (users[ID]->flags.hidden_sysop)))
      { continue; }
      message(r,text,0); 
    }
  }
}

void sendall(int ID, char *text)
{
int r;

  if (server_info->logfp!=NULL) 
  { 
#ifdef LOG_TIMESTAMPS
    time_t t=time(NULL);
    char stamp[32];
    ctime_r(&t,stamp);
    stamp[strlen(stamp)-1]=0;
    fprintf(server_info->logfp,"%s> %s",stamp,text); 
#else
    fprintf(server_info->logfp,"%s",text); 
#endif
    fflush(server_info->logfp);
  }

  for (r=0; r<server_info->max_conn; r++)
  {
    if (users[r]->inuse==1)
    {
      if (ID==-1) message(r,text,0);
	else
      if (NOT_GAGGED(r,ID) && !users[r]->flags.system_messages)
      {
        if (users[r]->user_level<10 && (users[ID]->user_level==10 &&
            users[ID]->flags.hidden_sysop))
        { continue; }

        message(r,text,0);
      }
    }
  }
}

void sendallbut(int ID, char *text)
{
int r;

  if (server_info->logfp!=NULL) 
  { 
#ifdef LOG_TIMESTAMPS
    time_t t=time(NULL);
    char stamp[32];
    ctime_r(&t,stamp);
    stamp[strlen(stamp)-1]=0;
    fprintf(server_info->logfp,"%s> %s",stamp,text); 
#else
    fprintf(server_info->logfp,"%s",text); 
#endif
    fflush(server_info->logfp);
  }

  for (r=0; r<server_info->max_conn; r++)
  {
    if (users[r]->inuse==1 && r!=ID)
    {
      if (ID==-1) message(r,text,0);
	else
      if (NOT_GAGGED(r,ID) && !users[r]->flags.system_messages)
      {
        if (users[r]->user_level<10 && (users[ID]->user_level==10 && users[ID]->flags.hidden_sysop))
        { continue; }
        message(r,text,0);
      }
    }
  }
}


