/**
 *  Naken Chat
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPL
 *
 * Copyright 1998-2013 by Michael Kohn
 *
 */

#ifndef NAKEN_PLUGIN_H
#define NAKEN_PLUGIN_H

#include "user.h"

typedef void (*message_t)(int ID, char *text, int hb_flag);
typedef void (*sendpriv_t)(int ID_TO, int ID_FROM, char *text);
typedef void (*sendchan_t)(int ID, int channel, char *text);
typedef void (*sendchanbut_t)(int ID, int channel, char *text);
typedef void (*sendall_t)(int ID, char *text);
typedef void (*sendallbut_t)(int ID, char *text);
typedef void (*kill_user_t)(int ID);

struct naken_context_t
{
  struct _user **users;
  message_t message;         // Send message to user ID
  sendpriv_t sendpriv;       // Send private message to user from user
  sendchan_t sendchan;       // Send message to everyone in user ID's channel
  sendchanbut_t sendchanbut; // Send message to all in channel but user ID
  sendall_t sendall;         // Send to everyone on the chat
  sendallbut_t sendallbut;   // Send to everyone on chat but user ID
  kill_user_t kill_user;      // Kick user ID off the chat
};


#ifdef WINDOWS
#define dlsym GetProcAddress
#define dlopen(libname,x) LoadLibrary(libname)
#define dlclose(libhandle) FreeLibrary(libhandle)
#endif

typedef int (*naken_init_t)(struct naken_context_t *naken_context);
typedef int (*naken_dot_command_t)(int ID, const char *text);
typedef int (*naken_public_text_t)(int ID, char *text);
typedef int (*naken_output_filter_t)(char *text);

struct naken_plugins_t
{
  /* These functions could be separated into other linked lists to
     save CPU cycles... that way if a function isn't implemented
     it won't be in the list... */

  // naken_init_t naken_init;
  naken_dot_command_t naken_dot_command;
  naken_public_text_t naken_public_text;
  naken_output_filter_t naken_output_filter;
#ifndef WINDOWS
  void *dlhandle;
#else
  HMODULE dlhandle;
#endif
  struct naken_plugins_t *next_plugin;
};

#endif


