/**
 *  Naken Chat
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPL
 *
 * Copyright 1998-2013 by Michael Kohn
 *
 */

#ifndef SERVER_INFO_H
#define SERVER_INFO_H

#include <stdint.h>

#ifdef ENABLE_SSL
#include <openssl/ssl.h>
#include <openssl/err.h>
#endif

#define MAXCHAN 20
#define CHANNEL_NAME_LEN 20
#define BAD_WORD_BUFFER_SIZE 1024
#define BANS_BUFFER_SIZE 4096
#define ALLOWS_BUFFER_SIZE 4096
#define NO_OWNER_BUFFER_SIZE 2048

struct _server_info
{
  int max_conn;
  int min_conn;
  int gag_size;
  FILE *logfp;
  char register_level;
  int debug;

  // Site restriction and censoring
  char allows[ALLOWS_BUFFER_SIZE];
  char bans[BANS_BUFFER_SIZE];
  char bad_word[BAD_WORD_BUFFER_SIZE];
  int allow_count;    // FIXME - Get rid of this
  int ban_count;      // FIXME - Get rid of this
  int bad_word_count; // FIXME - Get rid of this
  int censor_level;

  struct
  {
    uint32_t channel_disallow:1;
    uint32_t host_lookups:1;
    uint32_t show_ips:1;
    uint32_t no_dup_ips:1;
    uint32_t elite_mode:1;
    uint32_t force_password:1;
    uint32_t force_names:1;
    uint32_t no_dup_names:1;
    uint32_t no_name_changing:1;
    uint32_t kick_dups:1;
    uint32_t name_register:1;
    uint32_t international:1;
  } flags;

  // Channel info
  int channel_op[MAXCHAN];
  char channels[MAXCHAN][CHANNEL_NAME_LEN];
  char no_owner[NO_OWNER_BUFFER_SIZE];
  int channel_flags[MAXCHAN];

  // Welcome message
  char welcome[2048];
  int welcome_len;

  time_t max_idle_time;
  time_t uptime;

  int login_speed;
  int port;

  // Password used to .P into level 10
  char sysop_pw[15];

  // File paths
  char passwd[256];
  char logfile[256];
  char config_dir[256];
#ifdef ENABLE_SSL
  char ssl_cert[256];
  int ssl_port;
  SSL_CTX *ssl_ctx;
  //const SSL_METHOD *ssl_method;
#endif

#ifdef WHOFILE
  char whofile[256];
#ifdef WINDOWS
  CRITICAL_SECTION win_mutex;
#else
  pthread_mutex_t pthread_mutex;
  pthread_mutexattr_t pthread_mutexattr;
#endif
#endif
};

#endif

